/*
    $Id: aladindump.c,v 1.17 2003/01/12 13:24:37 cramer Exp $

    Converts Aladin raw files produced by aladinlink into UDCF format.

    Copyright (C) 2002 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  

*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <time.h>
#include <popt.h>

#include "version.h"
#include "config.h"
#include "aladin.h"

#define DEBUG (0) 

/* Function prototypes */
void create_aladin_dir(void);
void write_buffer(char *buf, int size, time_t dtime);
void usage (poptContext optCon, int exitcode, char *error, char *addl);

extern int errno;
static int checksum(char *, char verbose);

int main(int argc, const char **argv) {
  int fd;
  FILE * pf;
  char filename[255];

  /* Command line parsing arguments */
  char c;
  poptContext optCon;
  char debug=0;
  char verbose=0;
  
  const struct poptOption optionsTable[] = {
  		{ "verbose", 'v', POPT_ARG_NONE, 0, 'v', "verbose output", "v"   },
  		{ "debug",   'd', POPT_ARG_NONE, 0, 'd', "debugging output", "d" },
  		POPT_AUTOHELP
  		{ NULL, '\0', 0, NULL, 0 }
  };
 
  char aladinbuf[2048];
  char profilebuf[1536];
  int i,j,k, profilebuflength;
  int pbend;
  int rb;
  struct tm * dlt;
  int baseaddress;
  time_t dumptime, temp_time;
  time_t deltat;
  
  struct s_aladinsettings aladinsettings;
  struct s_aladinstatus aladinstatus;
  struct s_aladinprofile profile[37];
  struct s_aladinlogbook logbook[37];
  struct stat dumpstat;
  struct s_config config;
  t_aladin * aladin;

  optCon = poptGetContext(NULL, argc, argv, optionsTable, 0);
  /* poptSetOtherOptionHelp(optCon, "[OPTIONS]* <port>"); */

  printf("aladindump V%s %s\n", VERSION, COPY);
  readconfig(&config);

  /* Now do options processing */
  while ((c = poptGetNextOpt(optCon)) >= 0) {
    switch (c) {
      case 'v':
        verbose=1;
        break;
      case 'd':
        debug=1;
        break;
    }
  }
  
  if (c < -1) {
     /* an error occurred during option processing */
     fprintf(stderr, "%s: %s\n",
     poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
     poptStrerror(c));
     return 1;
  }

  printf("use --help for help\n");
  
  if (stat("aladindump.raw", &dumpstat) != 0) {
    printf("%s (%d): File 'aladindump.raw' does not exist.\n", strerror(errno), errno);
    exit(1);
  }
  dumptime = dumpstat.st_mtime;

  if (dumpstat.st_size != BUFSIZE) {
    printf("The file 'aladindump.raw' has the wrong size.");
    return(1);
  }

  fd = open("aladindump.raw", O_RDONLY);
  rb = read(fd, aladinbuf, BUFSIZE);
  close(fd);
  
  if (rb != BUFSIZE) {
    printf("Error while rading file. File has the wrong size.\n");
    return(1);
  }

  if (!checksum(aladinbuf, verbose)) {
    return(1);
  }
  
  create_aladin_dir();
  write_buffer(aladinbuf, BUFSIZE, dumptime);
  
  /* Fill up the settings section */
  
  aladinsettings.model = aladinbuf[0x07bc];
  aladinsettings.is_air = (aladinbuf[0x07bc] &0x0f) % 4 == 0 ? 1 : 0;
  aladinsettings.is_nitrox = (aladinbuf[0x07bc] & 0xf0) == 0xf0 ? 1 : 0;
  aladinsettings.is_o2 = (aladinbuf[0x07bc] & 0xf0) == 0xa0 ? 1 : 0;

  aladinsettings.beep  = (aladinbuf[0x07d2] & 0x02) == 0x02 ? 1 : 0;
  aladinsettings.units = (aladinbuf[0x07d2] & 0x01) == 0x01 ? 1 : 0;

  aladinsettings.reserve = aladinbuf[0x07de];

  aladinsettings.brethwarning = aladinbuf[0x07eb];

  aladinsettings.serial = (aladinbuf[0x07ed] << 16) + 
                          (aladinbuf[0x07ee] << 8) + 
                          aladinbuf[0x07ef];

  /*  Fill up status section */
  
  aladinstatus.battery = (unsigned char) aladinbuf[0x07f0] * 100 / 256;
  aladinstatus.batterystatus = (aladinbuf[0x07f1] & 0x0f) == 0 ? 1 : 0;
  
  aladinstatus.totaldives = ((unsigned char) aladinbuf[0x07f2] << 8) + 
                            (unsigned char) aladinbuf[0x07f3];

  aladinstatus.logbookoffset = ((aladinbuf[0x07f4] +36) % 37) * 12 +0x0600;
  aladinstatus.numprofiles = aladinbuf[0x07f5];
  aladinstatus.endringbuf = (((unsigned int)((unsigned char)aladinbuf[0x07f7] << 7) & 0xff00) 
                              | (unsigned char) aladinbuf[0x07f6]) & 0x07ff;

/*
  aladinstatus.rawtime = ((aladinbuf[0x07f8] << 24) + 
                            (aladinbuf[0x07f9] << 16) + 
                            (aladinbuf[0x07fa] << 8) + 
                            aladinbuf[0x07fb]) / 2;
*/

  aladinstatus.rawtime  = (unsigned char)aladinbuf[0x07f8] << 24;
  aladinstatus.rawtime += (unsigned char)aladinbuf[0x07f9] << 16;
  aladinstatus.rawtime += (unsigned char)aladinbuf[0x07fa] << 8;
  aladinstatus.rawtime += (unsigned char)aladinbuf[0x07fb];
  
  aladinstatus.rawtime = aladinstatus.rawtime / 2;

  /* display the values form the settings section */

  for (aladin = (t_aladin *)&aladintype; aladin->name; aladin++) {
    if (aladinsettings.model == aladin->type) {
      break;
    }
  }
  
  deltat = dumptime - aladinstatus.rawtime; 

  if (debug) {
    printf("****************** Time crrection info **************************\n");
    printf(" Internal Time counter at readouttime : %u\n", (int unsigned) aladinstatus.rawtime);
    printf(" Readout time of aladindump.raw is : %u\n", (unsigned int) dumptime);
    printf(" Deltat time correction factor : %u\n", (unsigned int) deltat);
  }

  if (verbose) {
    printf("===================== Aladin Status =============================\n");
    printf(" Battery Status : %d%% %s\n", aladinstatus.battery,
                           (aladinstatus.batterystatus == 1) ? "OK" : "Empty");
    printf(" Total Dives    : %d\n", aladinstatus.totaldives);

    printf(" Model: %s, Serial: %u\n", aladin->name, aladinsettings.serial);
    printf(" IS_AIR: %x, IS_NITROX: %x, IS_O2: %x\n", aladinsettings.is_air, 
                           aladinsettings.is_nitrox, aladinsettings.is_o2);
    printf(" Beep: %s, Unit: %s\n", aladinsettings.beep == 1 ? "ON" : "OFF", 
                           aladinsettings.units == 0 ? "Metric" : "Imperial");
    printf(" Reserve: %d Bar\n", aladinsettings.reserve);
    printf(" Brethwarning: %d\n", aladinsettings.brethwarning);
  }

  aladinstatus.time = aladinstatus.rawtime + deltat;
  dlt = localtime(&(aladinstatus.time));

  #if DEBUG > 0
  printf(" DEBUG: dumptime = %u, aladinrawtime = %u, deltat = %u\n", 
     (unsigned int) dumptime, (unsigned int) aladinstatus.rawtime, (unsigned int) deltat);
  printf(" DEBUG: aladintime = %d\n", (int) aladinstatus.time);
  #endif  
 
  if (verbose) {
    printf(" Data was downloaded on : %s\n", asctime(dlt));
  }

  /* and now to the logbook */
  
  i = aladinstatus.numprofiles;
  
  for (baseaddress = aladinstatus.logbookoffset; i > 0; baseaddress -= 12) {
    if (baseaddress < 0x0600) {
      baseaddress = 0x0600;
    }

    logbook[i].divenum = aladinstatus.totaldives - aladinstatus.numprofiles + i;
    logbook[i].altitude = ( aladinbuf[baseaddress] & 0xb0 ) >> 6;
    logbook[i].sos = ( aladinbuf[baseaddress] & 0x20 ) >> 5;
    logbook[i].workalarm = ( aladinbuf[baseaddress] & 0x10 ) >> 4;
    logbook[i].decoalarm = ( aladinbuf[baseaddress] & 0x08 ) >> 3;
    logbook[i].repeatd = ( aladinbuf[baseaddress] & 0x02 ) >> 1;
    logbook[i].acentwaring = aladinbuf[baseaddress] & 0x01;

    logbook[i].bottomtime = ((aladinbuf[baseaddress] & 0x04 ) >> 2) * 100 +
                            ((aladinbuf[baseaddress+1] & 0xf0) >> 4) * 10 + 
                            (aladinbuf[baseaddress+1] & 0x0f);

    /* Formula that strips out garbage in the last 6 bits */
    logbook[i].maxdepth = (((aladinbuf[baseaddress+2] << 8) + aladinbuf[baseaddress+3]) >>6) * 10.0 /64.0;

    /* Formula used as in ROM Program and Datatrak */
    /* 
      logbook[i].maxdepth = (((aladinbuf[baseaddress+2] << 8) + aladinbuf[baseaddress+3])) * 10.0 /4096.0;
    */
    
    if (logbook[i].repeatd) {
      logbook[i].surfaceh = ((aladinbuf[baseaddress+4] & 0xf0) >> 4) * 10 +
                            (aladinbuf[baseaddress+4] & 0x0f);          
                          
      logbook[i].surfacem = ((aladinbuf[baseaddress+5] & 0xf0) >> 4) * 10 +
                           (aladinbuf[baseaddress+5] & 0x0f);          
    } else {
      logbook[i].surfaceh = 0;
      logbook[i].surfacem = 0;
    }

    logbook[i].airconsumption = (unsigned char)aladinbuf[baseaddress+6] * 1.378951458634;
   
  
    logbook[i].entrytime = (((long long unsigned) (aladinbuf[baseaddress+7] << 24 )) +
    			   ((long long unsigned) (aladinbuf[baseaddress+8] << 16 )) +
                           ((long long unsigned) (aladinbuf[baseaddress+9] << 8  )) +
			   ((long long unsigned) ((char unsigned)aladinbuf[baseaddress+10]))) / 2
                           + deltat;
    

    logbook[i].watertemp = ((unsigned char) aladinbuf[baseaddress+11]) / 4.0;
    
    if (verbose) {
      printf("------------------------ Dive %d --------------------------------\n", 
                                logbook[i].divenum);
      printf("Altitude level    : %d\n", logbook[i].altitude);
      printf("Alarms            : SOS %d, WORK %d, DECO %d, ACENT %d\n", logbook[i].sos, 
                                logbook[i].workalarm, logbook[i].decoalarm, 
                                logbook[i].acentwaring);
      printf("Bottomtime        : %d\n", logbook[i].bottomtime);
      printf("Max. Depth        : %.2f\n", logbook[i].maxdepth);
      printf("Surface Interval  : %02d:%02d\n", logbook[i].surfaceh, logbook[i].surfacem);
      printf("Air consunption   : %.2f Bar\n", logbook[i].airconsumption);
      dlt = localtime(&(logbook[i].entrytime));
      printf("Entry Time        : %s", asctime(dlt));
      printf("Water Temperature : %.2f\n", logbook[i].watertemp);
    }
    i--;
  }
  
  /* and last, the dive profiles */
  
  if (debug) {
    printf("End of ringbuffer is at: %#x\n", aladinstatus.endringbuf);
  }

  /* first find the oldest logbook entry */
  for (baseaddress = aladinstatus.endringbuf; (unsigned char) aladinbuf[baseaddress] != 0xff; baseaddress++) {
    if (baseaddress > 0x5ff) {
      baseaddress = -1;
    }   
  }
 
  if (debug) {
  printf("Start of oldest diveprofile at: %#x, value at that position: %#x\n", 
                         baseaddress, (unsigned char) aladinbuf[baseaddress]);
  }

  if (debug) {
    printf("Copying ringbuffer content ... \n");
  }
  /* Copy profiles to a linear buffer */
  j=0;
  for (i=0; baseaddress+i != aladinstatus.endringbuf; i++) {
    if (baseaddress+i > 0x5ff) {
/*      baseaddress = -1; */
      baseaddress = 0;
      i=0;
    }
    profilebuf[j] = aladinbuf[baseaddress+i];
    j++;
  }
  if (debug) {
    printf("... done, copied %d bytes\n", j);
  }

  profilebuf[i] = aladinbuf[baseaddress+i];
  
  profilebuflength = i;
  
  pbend=j;
  baseaddress = 0;

  for (i=1; i <= aladinstatus.numprofiles; i++) {
    
    profile[i].divenum = aladinstatus.totaldives - aladinstatus.numprofiles + i;
    profile[i].ambienttemp = ((unsigned char) profilebuf[baseaddress+1]) / 4.0;

    #if DEBUG > 0
    printf("Dive Number        : %d\n", profile[i].divenum);
    printf("Ambient Temperatur : %.2f\n", profile[i].ambienttemp);
    #endif

    profile[i].tissue1 = profilebuf[baseaddress+2] + (profilebuf[baseaddress+2] << 8);
    profile[i].tissue2 = profilebuf[baseaddress+4] + (profilebuf[baseaddress+5] << 8);
    profile[i].tissue3 = profilebuf[baseaddress+6] + (profilebuf[baseaddress+7] << 8);
    profile[i].tissue4 = profilebuf[baseaddress+8] + (profilebuf[baseaddress+9] << 8);
    profile[i].tissue5 = profilebuf[baseaddress+10] + (profilebuf[baseaddress+11] << 8);
    profile[i].tissue6 = profilebuf[baseaddress+12] + (profilebuf[baseaddress+13] << 8);
    profile[i].tissue7 = profilebuf[baseaddress+14] + (profilebuf[baseaddress+15] << 8);
    profile[i].tissue8 = profilebuf[baseaddress+16] + (profilebuf[baseaddress+17] << 8);
    
    profile[i].microbubbles = profilebuf[baseaddress+18] + ((profilebuf[baseaddress+19] & 0xf0 ) << 4);
    profile[i].intrapulmenary = ((profilebuf[baseaddress+19] & 0x0f ) << 8) + profilebuf[baseaddress+20];

    profile[i].skincool = (unsigned int) (((profilebuf[baseaddress+21] + (profilebuf[baseaddress+22] & 0xf0)) << 4) >> 6);

    /* O2 and nitrox stiff */

    baseaddress+=22;
    
/*    
    printf("profilebuf[baseaddress+22] : %#x [%#x -> %d]\n", 
        (unsigned char) profilebuf[baseaddress], baseaddress, baseaddress);
*/

    if (aladinsettings.is_o2) {
      baseaddress += 3;
      printf("is_02\n");
    } else if (aladinsettings.is_nitrox) {
      baseaddress += 2;
      printf("is_nitrox\n");
    }
    
    j = 0;
    k = 0;
    
    baseaddress++;
  
    while (((unsigned char) profilebuf[baseaddress] != 0xff) && (baseaddress != pbend-1)) {
      profile[i].diveprofile[j] = (((unsigned int)(profilebuf[baseaddress] << 8) + 
                (unsigned char)profilebuf[baseaddress+1]) >> 6) * 0.15625;

      #if DEBUG >0
      printf("Depth[%d] = %f (%#x %#x)\n", j, profile[i].diveprofile[j], 
         (unsigned char)profilebuf[baseaddress], (unsigned char)profilebuf[baseaddress+1] );
      #endif
      
      profile[i].warnings[j].transmit = (profilebuf[baseaddress+1] & 0x20) >> 5;
      profile[i].warnings[j].work = (profilebuf[baseaddress+1] & 0x10) >> 4;
      profile[i].warnings[j].decomissed = (profilebuf[baseaddress+1] & 0x08) >> 3;
      profile[i].warnings[j].ascent = (profilebuf[baseaddress+1] & 0x04) >> 2;
      profile[i].warnings[j].bottom = (profilebuf[baseaddress+1] & 0x02) >> 1;
      profile[i].warnings[j].decostop = profilebuf[baseaddress+1] & 0x01;
      
      baseaddress += 2;

      if (((unsigned char) profilebuf[baseaddress] != 0xff) && (j % 3 == 2)) {
      
        /* printf("i j k = %d %d %d\n", i, j, k); */
        
        profile[i].decoinfo[k].physeffort = (profilebuf[baseaddress] & 0x70) >> 4;

        /* 
        Have to look at this ...
          profile[i].decoinfo[k].skincooling;
        */
      
        profile[i].decoinfo[k].microbubble = profilebuf[baseaddress] & 0x07;
      
        baseaddress++;
        k++;
      }
      
      if (aladinsettings.is_o2) {
        printf("is_02\n");
        baseaddress++;
      }
      j++;
    }
    profile[i].profilesamples = j;  
  }


  printf("\n");
  if (debug) {
    printf("Newest Logbook Entry at: %#x (%u), End of Buffer: %#x\n", 
                  aladinstatus.logbookoffset, (unsigned char) aladinbuf[0x07f4], 
                  aladinstatus.endringbuf);
  }
  
  /* write the data out in UDCF format */

  for(i=1; i <= aladinstatus.numprofiles; i++) {
  
    temp_time = logbook[i].entrytime;

    dlt = localtime(&temp_time);

    snprintf(filename, 255, "%s/%s/%d-%04d-%02d-%02d_%02d%02d.udcf",
                      getenv("HOME"), 
                      ALADINHOME,
                      logbook[i].divenum+config.aladindiveoffset, 
                      dlt->tm_year+1900, dlt->tm_mon+1, 
                      dlt->tm_mday, dlt->tm_hour, dlt->tm_min);
    if (open(filename,O_RDONLY) == -1) {
      pf = fopen(filename,"w");
      if (pf == NULL) {
        fprintf(stderr, "Error: Can not open UDCF file!\n");
        return(1);
      }
      printf("Dive will be saved in UDCF format as %s\n", filename);
      
      fprintf(pf, "<PROFILE UDCF=\"1\">\n");
      fprintf(pf, "<UNITS>Metric</UNITS>\n");
      fprintf(pf, "<DEVICE>\n <VENDOR>UWATEC</VENDOR>\n <MODEL>%s</MODEL>\n <VERSION>1</VERSION>\n</DEVICE>\n", 
                        aladin->name);
      fprintf(pf, "<ADDINFO>\n");
      fprintf(pf, " <PERSINFO>%s</PERSINFO>\n", config.aladinowner);
      fprintf(pf, " <TOTALDIVES>%d</TOTALDIVES>\n", logbook[i].divenum+config.aladindiveoffset);
      fprintf(pf, " <SERIALID>%d</SERIALID>\n", aladinsettings.serial);   
      fprintf(pf, "</ADDINFO>\n");
      fprintf(pf, "<REPGROUP>\n");
      fprintf(pf, " <DIVE>\n");
      fprintf(pf, "  <DATE><DAY>%d</DAY><MONTH>%d</MONTH><YEAR>%04d</YEAR></DATE>\n", dlt->tm_mday, dlt->tm_mon+1, dlt->tm_year+1900);
      fprintf(pf, "  <TIME><HOUR>%d</HOUR><MINUTE>%d</MINUTE></TIME>\n", dlt->tm_hour, dlt->tm_min);
      fprintf(pf, "  <SURFACEINTERVAL>%d</SURFACEINTERVAL>\n", logbook[i].surfaceh * 60 + logbook[i].surfacem);
      fprintf(pf, "  <TEMPERATURE>%.2f</TEMPERATURE>\n", logbook[i].watertemp);
      fprintf(pf, "  <DENSITY>1030.000</DENSITY>\n");
      fprintf(pf, "  <ALTITUDE>%d</ALTITUDE>\n", aladinaltitudes[logbook[i].altitude]);
      fprintf(pf, "  <GASES>\n");                            
      fprintf(pf, "   <MIX>\n");
      fprintf(pf, "    <MIXNAME>RealAir</MIXNAME>\n");   
      fprintf(pf, "    <TANK><PDELTA>%.0f</PDELTA></TANK>\n", logbook[i].airconsumption * 100000);
/*
      fprintf(pf, "    <TANKVOLUME>15.0</TANKVOLUME>\n");
      fprintf(pf, "    <TANK><PSTART>%d00000.0</PSTART><PEND>%d00000.0</PEND></TANK>\n", dive[i].startpresure, dive[i].endpresure);
*/      
      fprintf(pf, "    <O2>0.21</O2>\n");
      fprintf(pf, "    <N2>0.79</N2>\n");
      fprintf(pf, "    <HE>0.00</HE>\n");
      fprintf(pf, "   </MIX>\n");        
      fprintf(pf, "  </GASES>\n");
      fprintf(pf, "  <DELTAMODE/>\n");
      fprintf(pf, "  <SAMPLES>\n");
      fprintf(pf, "   <DELTA>%d</DELTA>\n", 20);
      fprintf(pf, "   <SWITCH>RealAir</SWITCH>\n");
      
      fprintf(pf, "   <D>0.0</D>\n");


//      k = (logbook[i].bottomtime * 3) < profile[i].profilesamples ? logbook[i].bottomtime * 3 : profile[i].profilesamples;
//      for (j=0; j<=k; j++) {

      for (j=0; j<=profile[i].profilesamples; j++) {

        if (profile[i].warnings[j].decostop) {
          fprintf(pf, "   <ALARM>DECO</ALARM>\n");
        }

        if (profile[i].warnings[j].work) {
          fprintf(pf, "   <ALARM>WORK</ALARM>\n");
        }

        if (profile[i].warnings[j].decomissed) {
          fprintf(pf, "   <ALARM>DECO MISSED</ALARM>\n");
        }

        if (profile[i].warnings[j].ascent) {
          fprintf(pf, "   <ALARM>ASCENT</ALARM>\n");
        }

        if (profile[i].warnings[j].bottom) {
          fprintf(pf, "   <ALARM>BOTTOMTIME</ALARM>\n");
        }

        if (profile[i].warnings[j].transmit) {
          fprintf(pf, "   <ALARM>TRANSMIT</ALARM>\n");
        }
       
        fprintf(pf, "   <D>%6.2f</D>\n", profile[i].diveprofile[j]);

      }
      
      fprintf(pf, "  </SAMPLES>\n");
      fprintf(pf, " </DIVE>\n");
      fprintf(pf, "</REPGROUP>\n");
      fprintf(pf, "</PROFILE>\n");

      fclose(pf);
    } else {
      printf("Dive '%s' already exists, skipping [%s (%d)] ...\n", filename, strerror(errno), errno);
    }
  }

  return(0);  
}  /* end main */


static int checksum(char * buf, char verbose) {
  int i;
  unsigned int csum = 0;
  unsigned int filecsum;
  
  filecsum = (((unsigned char) buf[0x07fd] << 8) | (unsigned char) buf[0x07fc]);
  /* filecsum &= 0x0ffff; */

  if (verbose) {
    printf("File Checksum %#04x ... ", filecsum);
  }

  for (i=0; i < 0x07fc; i++) {
    csum += (unsigned char) buf[i];
  }

  csum += 0x1fe;
  csum %= 65536;
  csum &= 0x0ffff;

  if ( csum == filecsum) {
    if (verbose) {
      printf("OK\n");
    }
    return(1);
  } else {
    if (verbose) {
      printf("Error\n");
    }
    return(0);
  }
}

/* create aladin direcrory structure in users homedir */
void create_aladin_dir() {
  char dirname[100];
  struct stat dirstat;

  /* Look if ".aladin" does exist */
  snprintf(dirname, 100, "%s/%s/", getenv("HOME"), ALADINHOME);
    
  stat(dirname, &dirstat);
    
  if ((dirstat.st_mode & S_IFDIR) != S_IFDIR) {
    printf("Create .aladin directory ...");
    mkdir(dirname, 0755);
    snprintf(dirname, 100, "%s/%s/archive",getenv("HOME"), ALADINHOME);
    mkdir(dirname, 0755);
    printf(" done\n");
  }
}

/* write buffer to archive file */
void write_buffer(char *buf, int size, time_t dtime) {
  char filename[255];
  int fd;
  int wsize;
  
  snprintf(filename, 255, "%s/%s/archive/aladindump-%d.raw", getenv("HOME"), ALADINHOME, (int) dtime);
  
  #if DEBUG > 0
  printf("Archive file is: %s\n", filename);
  #endif
  
  fd = open(filename, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
  if (fd < 0) {
     printf("Cound not open %s for writing!", filename);
     exit(1);
  }
  wsize = write(fd, buf, size);
  close(fd); 
  
  if (wsize != size) {
     printf("Could not write archive file correctly!\n");
     exit(1);
  }
}

void usage(poptContext optCon, int exitcode, char *error, char *addl) {
  poptPrintUsage(optCon, stderr, 0);
  if (error) fprintf(stderr, "%s: %s0", error, addl);
  exit(exitcode);
}

